/*
 * Copyright (C) 2012, Nils Asmussen <nils@os.inf.tu-dresden.de>
 * Economic rights: Technische Universitaet Dresden (Germany)
 *
 * This file is part of NRE (NOVA runtime environment).
 *
 * NRE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * NRE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License version 2 for more details.
 */

#include <stream/Serial.h>
#include <kobj/Pt.h>
#include <CPU.h>

using namespace nre;

static Pt *echo;

PORTAL static void portal_getcap(capsel_t) {
    UtcbFrameRef uf;
    uf.delegate(echo->sel());
}

PORTAL static void portal_echo(capsel_t) {
    UtcbFrameRef uf;
    int x;
    uf >> x;
    uf.finish_input();
    uf << x;
}

// provides the echo portal
static void service() {
    LocalThread *lt = LocalThread::create(CPU::current().log_id());
    echo = new Pt(lt, portal_echo);
}

// wants to call the echo portal
static void client(Pt &getcap) {
    capsel_t cap = CapSelSpace::get().allocate(1);
    {
        UtcbFrame uf;
        uf.delegation_window(Crd(cap,0,Crd::OBJ_ALL));
        getcap.call(uf);
    }

    // call portal
    int res;
    Pt echopt(cap);
    UtcbFrame uf;
    uf << 42;
    echopt.call(uf);
    uf >> res;

    Serial::get() << "Hello World: " << res << "\n";
}

int main() {
    service();

    // given from someone else
    LocalThread *lt1 = LocalThread::create(CPU::current().log_id());
    Pt getcap(lt1, portal_getcap);

    client(getcap);
    return 0;
}
